@@ -145,7 +145,6 @@ static void SpellMoveToTarget(CUnit *unit)
 */
 void HandleActionSpellCast(CUnit *unit)
 {
-	int flags;
 	const SpellType *spell;
 
 	if (unit->Wait) {
@@ -165,9 +164,9 @@ void HandleActionSpellCast(CUnit *unit)
 				//
 				// Notify player about this problem
 				//
-				if (unit->Variable[MANA_INDEX].Value < spell->ManaCost) {
+				if (unit->Variable[CHARGE_INDEX].Value < spell->ChargeCost) {
 					unit->Player->Notify(NotifyYellow, unit->X, unit->Y,
-						_("%s: not enough mana for spell: %s"),
+						_("%s: not charged enough for this action: %s"),
 						unit->Type->Name.c_str(), spell->Name.c_str());
 				} else {
 					unit->Player->Notify(NotifyYellow, unit->X, unit->Y,
@@ -201,7 +200,7 @@ void HandleActionSpellCast(CUnit *unit)
 		case 2:                         // Cast spell on the target.
 			// FIXME: should use AnimateActionSpellCast here
 			if (unit->Type->Animations->Attack) {
-				flags = UnitShowAnimation(unit, unit->Type->Animations->Attack);
+				UnitShowAnimation(unit, unit->Type->Animations->Attack);
 				if (unit->Anim.Unbreakable) { // end of animation
 					return;
 				}
