@@ -252,11 +252,6 @@ static void InitGameCallbacks(void)
 */
 void GameMainLoop(void)
 {
-#ifdef DEBUG  // removes the setjmp warnings
-	static bool showtip;
-#else
-	bool showtip;
-#endif
 	int player;
 	const EventCallback *old_callbacks;
 
@@ -271,8 +266,6 @@ void GameMainLoop(void)
 
 	CParticleManager::init();
 
-	showtip = false;
-
 	MultiPlayerReplayEachCycle();
 
 	CclCommand("GameStarting()");
@@ -297,7 +290,7 @@ void GameMainLoop(void)
 			//
 			// Work todo each second.
 			// Split into different frames, to reduce cpu time.
-			// Increment mana of magic units.
+			// Increment charge of spell casting units.
 			// Update mini-map.
 			// Update map fog of war.
 			// Call AI.
