@@ -177,7 +177,7 @@ static FullReplay *StartReplay(void)
 
 	replay->Comment1 = "Generated by Bos Wars Version " VERSION "";
 	replay->Comment2 = "Visit http://www.boswars.org for more information";
-	replay->Comment3 = "$Id: replay.cpp 9766 2010-04-21 08:26:37Z jim4 $";
+	replay->Comment3 = "$Id$";
 
 	if (GameSettings.NetGameType == SettingsSinglePlayerGame) {
 		replay->Type = ReplaySinglePlayer;
@@ -763,7 +763,7 @@ static void DoNextReplay(void)
 #ifdef DEBUG
 		if (!ReplayStep->SyncRandSeed) {
 			// Replay without the 'sync info
-			ThisPlayer->Notify(NotifyYellow, -1, -1, _("No sync info for this replay !"));
+			ThisPlayer->Notify(NotifyYellow, -1, -1, "%s", _("No sync info for this replay !"));
 		} else {
 			ThisPlayer->Notify(NotifyYellow, -1, -1, _("Replay got out of sync (%lu) !"), GameCycle);
 			DebugPrint("OUT OF SYNC %u != %u\n" _C_ SyncRandSeed _C_ ReplayStep->SyncRandSeed);
@@ -774,7 +774,7 @@ static void DoNextReplay(void)
 			// return;
 		}
 #else
-		ThisPlayer->Notify(NotifyYellow, -1, -1, _("Replay got out of sync !"));
+		ThisPlayer->Notify(NotifyYellow, -1, -1, "%s", _("Replay got out of sync !"));
 		ReplayStep = 0;
 		NextLogCycle = ~0UL;
 		return;
@@ -872,7 +872,7 @@ static void ReplayEachCycle(void)
 	}
 
 	if (!ReplayStep) {
-		SetMessage(_("End of replay"));
+		SetMessage("%s", _("End of replay"));
 		GameObserve = false;
 		return;
 	}
@@ -886,7 +886,7 @@ static void ReplayEachCycle(void)
 	} while (ReplayStep && (NextLogCycle == ~0UL || NextLogCycle == GameCycle));
 
 	if (!ReplayStep) {
-		SetMessage(_("End of replay"));
+		SetMessage("%s", _("End of replay"));
 		GameObserve = false;
 	}
 }
