@@ -57,8 +57,9 @@
 **    ::MapFieldLandAllowed land units are allowed.
 **    ::MapFieldCoastAllowed coast units (transporter) and coast
 **      buildings (shipyard) are allowed.
-**    ::MapFieldWaterAllowed water units allowed.
-**    ::MapFieldNoBuilding no buildings allowed.
+**    ::MapFieldShallowWater small water units allowed.
+**    ::MapFieldDeepWater both small and big water units allowed.
+**    ::MapFieldNoBuilding no buildings allowed, except neutral (e.g. rocks).
 **    ::MapFieldUnpassable field is movement blocked.
 **    ::MapFieldLandUnit land unit on field.
 **    ::MapFieldAirUnit air unit on field.
@@ -165,19 +166,25 @@ public:
 
 #define MapFieldSpeedMask    0x0007  /// Move faster on this tile
 
+#define MapFieldShallowWater 0x0008  /// Small water units allowed
 #define MapFieldLandAllowed  0x0010  /// Land units allowed
 #define MapFieldCoastAllowed 0x0020  /// Coast (transporter) units allowed
-#define MapFieldWaterAllowed 0x0040  /// Water units allowed
-#define MapFieldNoBuilding   0x0080  /// No buildings allowed
+#define MapFieldDeepWater    0x0040  /// Small and big water units allowed
+#define MapFieldNoBuilding   0x0080  /// No buildings allowed, except neutral (e.g. rocks)
 
 #define MapFieldUnpassable   0x0100  /// Field is movement blocked
 #define MapFieldTransparent  0x0200  /// Field is transparent
 
+#define MapFieldPatchMask    0x01FF  /// Flags copied from patch to map field
+
 #define MapFieldLandUnit     0x1000  /// Land unit on field
 #define MapFieldAirUnit      0x2000  /// Air unit on field
 #define MapFieldSeaUnit      0x4000  /// Water unit on field
 #define MapFieldBuilding     0x8000  /// Building on field
 
+#define MapFieldNormalSpeed  3       /// Normal CMapField::Flags & MapFieldSpeedMask
+#define MapFieldNormalCost   (1 << MapFieldNormalSpeed) /// Normal CMapField::Cost
+
 /*----------------------------------------------------------------------------
 --  Map info structure
 ----------------------------------------------------------------------------*/
