@@ -9,7 +9,7 @@
 //
 /**@name minimap.cpp - The minimap. */
 //
-//      (c) Copyright 1998-2008 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2010 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -80,8 +80,10 @@ static int Map2MinimapY[MaxMapHeight];     /// fast conversion table
 // MinimapScale:
 // 32x32 64x64 96x96 128x128 256x256 512x512 ...
 // *4 *2 *4/3   *1 *1/2 *1/4
-static int MinimapScaleX;                  /// Minimap scale to fit into window
-static int MinimapScaleY;                  /// Minimap scale to fit into window
+static int MinimapScaleX;   /// Minimap scale to fit into window rounded up
+static int MinimapScaleY;   /// Minimap scale to fit into window rounded up
+static int lowMinimapScaleX;   /// Minimap scale to fit into window rounded down
+static int lowMinimapScaleY;   /// Minimap scale to fit into window rounded down
 
 #define MAX_MINIMAP_EVENTS 8
 
@@ -124,9 +126,11 @@ void CMinimap::Create(void)
 
 	MinimapScaleX = (W * MINIMAP_FAC + maxSize - 1) / maxSize;
 	MinimapScaleY = (H * MINIMAP_FAC + maxSize - 1) / maxSize;
+	lowMinimapScaleX = (W * MINIMAP_FAC) / maxSize;
+	lowMinimapScaleY = (H * MINIMAP_FAC) / maxSize;
 
-	XOffset = (W - (Map.Info.MapWidth * MinimapScaleX) / MINIMAP_FAC + 1) / 2;
-	YOffset = (H - (Map.Info.MapHeight * MinimapScaleY) / MINIMAP_FAC + 1) / 2;
+	XOffset = (W - (Map.Info.MapWidth * lowMinimapScaleX) / MINIMAP_FAC + 1) / 2;
+	YOffset = (H - (Map.Info.MapHeight * lowMinimapScaleY) / MINIMAP_FAC + 1) / 2;
 
 	//
 	// Calculate minimap fast lookup tables.
@@ -142,10 +146,10 @@ void CMinimap::Create(void)
 		Minimap2MapY[n] = (((n - YOffset) * MINIMAP_FAC) / MinimapScaleY);
 	}
 	for (n = 0; n < Map.Info.MapWidth; ++n) {
-		Map2MinimapX[n] = (n * MinimapScaleX) / MINIMAP_FAC;
+		Map2MinimapX[n] = (n * lowMinimapScaleX) / MINIMAP_FAC;
 	}
 	for (n = 0; n < Map.Info.MapHeight; ++n) {
-		Map2MinimapY[n] = (n * MinimapScaleY) / MINIMAP_FAC;
+		Map2MinimapY[n] = (n * lowMinimapScaleY) / MINIMAP_FAC;
 	}
 
 	if (!UseOpenGL) {
@@ -198,7 +202,7 @@ CColor GetColor(CPatch *patch, int xoffset, int yoffset, int mx, int my, int sca
 	int x = (xoffset * TileSizeX + 7 + ((mx * SCALE_PRECISION) % scalex) / SCALE_PRECISION * 8);
 	int y = (yoffset * TileSizeY + 6 + ((my * SCALE_PRECISION) % scaley) / SCALE_PRECISION * 8);
 
-	Uint8 r, g, b, a;
+	Uint8 r = 0, g = 0, b = 0, a = 0;
 	SDL_Surface *s = patch->getType()->getGraphic()->Surface;
 	SDL_PixelFormat *f = s->format;
 
@@ -217,7 +221,7 @@ CColor GetColor(CPatch *patch, int xoffset, int yoffset, int mx, int my, int sca
 			r = pixel[f->Rshift >> 3];
 			g = pixel[f->Gshift >> 3];
 			b = pixel[f->Bshift >> 3];
-			a = 0;
+			a = 0xFF;
 			break;
 		case 4:
 			SDL_GetRGBA(*(Uint32 *)pixel, f, &r, &g, &b, &a);
@@ -487,7 +491,12 @@ static void DrawEvents()
 void CMinimap::Draw(int vx, int vy)
 {
 	if (!UseOpenGL) {
-		SDL_Rect drect = {X, Y, 0, 0};
+		SDL_Rect drect = {
+			static_cast<Sint16>(X),
+			static_cast<Sint16>(Y),
+			0, // SDL_BlitSurface ignores the width and height.
+			0
+		};
 		SDL_BlitSurface(MinimapSurface, NULL, TheScreen, &drect);
 	} else {
 		glBindTexture(GL_TEXTURE_2D, MinimapTexture);
@@ -579,10 +588,10 @@ void CMinimap::Destroy()
 void CMinimap::DrawCursor(int vx, int vy)
 {
 	// Determine and save region below minimap cursor
-	int x = X + XOffset + (vx * MinimapScaleX) / MINIMAP_FAC;
-	int y = Y + YOffset + (vy * MinimapScaleY) / MINIMAP_FAC;
-	int w = (UI.SelectedViewport->MapWidth * MinimapScaleX) / MINIMAP_FAC;
-	int h = (UI.SelectedViewport->MapHeight * MinimapScaleY) / MINIMAP_FAC;
+	int x = X + XOffset + (vx * lowMinimapScaleX) / MINIMAP_FAC;
+	int y = Y + YOffset + (vy * lowMinimapScaleY) / MINIMAP_FAC;
+	int w = (UI.SelectedViewport->MapWidth * lowMinimapScaleX) / MINIMAP_FAC;
+	int h = (UI.SelectedViewport->MapHeight * lowMinimapScaleY) / MINIMAP_FAC;
 
 	// Draw cursor as rectangle (Note: unclipped, as it is always visible)
 	Video.DrawTransRectangle(UI.ViewportCursorColor, x, y, w, h, 128);
@@ -602,8 +611,8 @@ void CMinimap::AddEvent(int x, int y)
 
 	MinimapEvent *minimapEvent = &MinimapEvents[NumMinimapEvents++];
 
-	minimapEvent->X = X + XOffset + (x * MinimapScaleX) / MINIMAP_FAC;
-	minimapEvent->Y = Y + YOffset + (y * MinimapScaleY) / MINIMAP_FAC;
+	minimapEvent->X = X + XOffset + (x * lowMinimapScaleX) / MINIMAP_FAC;
+	minimapEvent->Y = Y + YOffset + (y * lowMinimapScaleY) / MINIMAP_FAC;
 	minimapEvent->Size = (W < H) ? W / 3 : H / 3;
 }
 
