@@ -387,7 +387,9 @@ unsigned char *CNetworkPacket::Serialize(int numcommands) const
 
 	for (int i = 0; i < numcommands; ++i) {
 		if (this->Header.Type[i] == MessageExtendedCommand) {
-			((CNetworkExtendedCommand *)&this->Command[i])->Serialize(p);
+			CNetworkExtendedCommand *nec;
+			nec = (CNetworkExtendedCommand *)&Command[i];
+			nec->Serialize(p);
 		} else if (this->Header.Type[i] == MessageChat) {
 			((CNetworkChat *)&this->Command[i])->Serialize(p);
 		} else {
@@ -420,7 +422,9 @@ int CNetworkPacket::Deserialize(unsigned char *p, unsigned int len)
 
 	for (int i = 0; i < commands; ++i) {
 		if (this->Header.Type[i] == MessageExtendedCommand) {
-			((CNetworkExtendedCommand *)&this->Command[i])->Deserialize(p);
+			CNetworkExtendedCommand *nec;
+			nec = (CNetworkExtendedCommand *)&Command[i];
+			nec->Deserialize(p);
 		} else if (this->Header.Type[i] == MessageChat) {
 			((CNetworkChat *)&this->Command[i])->Deserialize(p);
 		} else {
@@ -1163,7 +1167,7 @@ static void ParseNetworkCommand(const CNetworkCommandQueue *ncq)
 			if (ply != NetworkSyncSeeds[GameCycle & 0xFF] ||
 					ntohs(ncq->Data.Unit) != NetworkSyncHashs[GameCycle & 0xFF]) {
 
-				SetMessage(_("Network out of sync"));
+				SetMessage("%s", _("Network out of sync"));
 				DebugPrint("\nNetwork out of sync %x!=%x! %d!=%d! Cycle %lu\n\n" _C_
 					ply _C_ NetworkSyncSeeds[GameCycle & 0xFF] _C_
 					ntohs(ncq->Data.Unit) _C_ NetworkSyncHashs[GameCycle & 0xFF] _C_ GameCycle);
@@ -1412,7 +1416,7 @@ void NetworkRecover(void)
 				NetworkIn[n & 0xFF][Hosts[i].PlyNr][0].Type = MessageQuit;
 				NetworkIn[n & 0xFF][Hosts[i].PlyNr][0].Data = nc;
 				PlayerQuit[Hosts[i].PlyNr] = 1;
-				SetMessage(_("Timed out"));
+				SetMessage("%s", _("Timed out"));
 
 				ncq = &NetworkIn[n & 0xFF][Hosts[i].PlyNr][0];
 				np.Header.Cycle = ncq->Time & 0xFF;
