@@ -225,7 +225,7 @@ std::string LocalPlayerName;         /// Name of local player
 
 	/// Name, Version, Copyright
 static std::string NameLine =
-	"Bos Wars V" VERSION ", (c) 1998-2013 by the Bos Wars and Stratagus Project.";
+	"Bos Wars V" VERSION ", (c) 1998-2023 by the Bos Wars and Stratagus Project.";
 
 std::string CliMapName;          /// Filename of the map given on the command line
 std::string CompileOptions;          /// Compile options.
@@ -342,6 +342,8 @@ static int MenuLoop()
 */
 void CleanGame(void)
 {
+	FastForwardCycle = 0;
+
 	EndReplayLog();
 	CleanMessages();
 
@@ -435,62 +437,6 @@ void StartReplay(const std::string &filename, bool reveal)
 	StartMap(CurrentMapPath, false);
 }
 
-/**
-**  Save the replay
-**
-**  @param filename  Name of the file to save to
-**
-**  @return          0 for success, -1 for failure
-*/
-int SaveReplay(const std::string &filename)
-{
-	FILE *fd;
-	char *buf;
-	std::ostringstream logfile;
-	std::string destination;
-	struct stat sb;
-
-	if (filename.find_first_of("\\/") != std::string::npos) {
-		fprintf(stderr, "\\ or / not allowed in SaveReplay filename\n");
-		return -1;
-	}
-
-	destination = UserDirectory + "logs/" + filename;
-
-	logfile << UserDirectory << "logs/log_of_stratagus_" << ThisPlayer->Index << ".log";
-
-	if (stat(logfile.str().c_str(), &sb)) {
-		fprintf(stderr, "stat failed\n");
-		return -1;
-	}
-	buf = new char[sb.st_size];
-	if (!buf) {
-		fprintf(stderr, "Out of memory\n");
-		return -1;
-	}
-	fd = fopen(logfile.str().c_str(), "rb");
-	if (!fd) {
-		fprintf(stderr, "fopen failed\n");
-		delete[] buf;
-		return -1;
-	}
-	fread(buf, sb.st_size, 1, fd);
-	fclose(fd);
-
-	fd = fopen(destination.c_str(), "wb");
-	if (!fd) {
-		fprintf(stderr, "Can't save to `%s'\n", destination.c_str());
-		delete[] buf;
-		return -1;
-	}
-	fwrite(buf, sb.st_size, 1, fd);
-	fclose(fd);
-
-	delete[] buf;
-
-	return 0;
-}
-
 //----------------------------------------------------------------------------
 
 /**
@@ -634,6 +580,7 @@ static void Usage(void)
 		"  -D bpp        Video mode depth = bits per pixel (for Win32/TNT)\n"
 		"  -F            Full screen video mode\n"
 		"  -S speed      Video sync speed (100 = 30 frames/s)\n"
+		"  -f cycle      Fast forward until the cycle number\n"
 		"  -W            Windowed video mode\n"
 		"Specifying MAP-PRESENTATION-FILE starts a single-player game in that map.\n"
 		"Example: boswars maps/islandwar.map/presentation.smp\n");
@@ -805,7 +752,7 @@ int main(int argc, char **argv)
 	//  Parse commandline
 	//
 	for (;;) {
-		switch (getopt(argc, argv, "c:d:ehlP:v:D:E:FL:S:U:W?")) {
+		switch (getopt(argc, argv, "c:d:ehlP:v:D:E:FL:S:f:U:W?")) {
 			case 'c':
 				CclStartFile = optarg;
 				continue;
@@ -870,7 +817,9 @@ int main(int argc, char **argv)
 			case 'S':
 				VideoSyncSpeed = atoi(optarg);
 				continue;
-
+			case 'f':
+				FastForwardCycle = atol(optarg);
+				continue;
 			case -1:
 				break;
 			case '?':
