@@ -51,6 +51,7 @@
 #include "spells.h"
 #include "sound.h"
 #include "map.h"
+#include "actions.h"
 #include "commands.h"
 #include "video.h"
 #include "font.h"
@@ -485,7 +486,7 @@ void UpdateStatusLineForButton(const ButtonAction *button)
 			SetCosts(0, UnitTypes[button->Value]->ProductionCosts);
 			break;
 		case ButtonSpellCast:
-			SetCosts(SpellTypeTable[button->Value]->ManaCost, NULL);
+			SetCosts(SpellTypeTable[button->Value]->ChargeCost, NULL);
 			break;
 		default:
 			ClearCosts();
@@ -507,7 +508,7 @@ void UpdateStatusLineForButton(const ButtonAction *button)
 **
 **  @todo FIXME: better check. (dependancy, resource, ...)
 */
-static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
+bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 {
 	bool res;
 
@@ -545,8 +546,11 @@ static bool IsButtonAllowed(const CUnit *unit, const ButtonAction *buttonaction)
 			}
 			break;
 		case ButtonTrain:
+			res = UnitIdAllowed(unit->Player, buttonaction->Value) != 0
+				&& TerrainAllowsTraining(unit, UnitTypes[buttonaction->Value]);
+			break;
 		case ButtonBuild:
-			res = true;
+			res = UnitIdAllowed(unit->Player, buttonaction->Value) != 0;
 			break;
 		case ButtonSpellCast:
 			res = SpellIsAvailable(unit->Player, buttonaction->Value);
