@@ -77,8 +77,6 @@
 --  Variables
 ----------------------------------------------------------------------------*/
 
-static int SavedMapPositionX[3];     /// Saved map position X
-static int SavedMapPositionY[3];     /// Saved map position Y
 static char Input[80];               /// line input for messages/long commands
 static int InputIndex;               /// current index into input
 static char InputStatusLine[99];     /// Last input status line
@@ -390,28 +388,6 @@ static void UiCenterOnSelected(void)
 	}
 }
 
-/**
-**  Save current map position.
-**
-**  @param position  Map position slot.
-*/
-static void UiSaveMapPosition(unsigned position)
-{
-	SavedMapPositionX[position] = UI.SelectedViewport->MapX;
-	SavedMapPositionY[position] = UI.SelectedViewport->MapY;
-}
-
-/**
-**  Recall map position.
-**
-**  @param position  Map position slot.
-*/
-static void UiRecallMapPosition(unsigned position)
-{
-	UI.SelectedViewport->Set(
-		SavedMapPositionX[position], SavedMapPositionY[position], TileSizeX / 2, TileSizeY / 2);
-}
-
 /**
 **  Toggle terrain display on/off.
 */
@@ -522,6 +498,7 @@ static bool CommandKey(int key)
 			return true;
 
 		// Unselect everything
+		case SDLK_ESCAPE:
 		case SDLK_CARET:
 		case SDLK_BACKQUOTE:
 			UiUnselectAll();
@@ -555,9 +532,9 @@ static bool CommandKey(int key)
 		case SDLK_F3:
 		case SDLK_F4: // Set/Goto place
 			if (KeyModifiers & ModifierShift) {
-				UiSaveMapPosition(key - SDLK_F2);
+				SetSavedMapPosition(key - SDLK_F2, UI.SelectedViewport->MapX, UI.SelectedViewport->MapY);
 			} else {
-				UiRecallMapPosition(key - SDLK_F2);
+				RecallSavedMapPosition(key - SDLK_F2);
 			}
 			break;
 
@@ -692,7 +669,7 @@ static bool CommandKey(int key)
 */
 int HandleCheats(const std::string &input)
 {
-	int ret;
+	int ret = 0;
 
 #ifdef DEBUG
 	if (input == "ai me") {
@@ -723,17 +700,18 @@ int HandleCheats(const std::string &input)
 	lua_gettable(Lua, LUA_GLOBALSINDEX);
 	if (!lua_isfunction(Lua, -1)) {
 		DebugPrint("No HandleCheats function in lua.\n");
-		return 0;
-	}
-	lua_pushstring(Lua, input.c_str());
-	LuaCall(1, 0, false);
-	if (lua_gettop(Lua) - base == 1) {
-		ret = LuaToBoolean(Lua, -1);
-		lua_pop(Lua, 1);
 	} else {
-		DebugPrint("HandleCheats must return a boolean");
-		ret = 0;
+		lua_pushstring(Lua, input.c_str());
+		if (LuaCall(1, 0, false)) {
+			// A Lua error occurred within HandleCheats,
+			// and LuaCall already reported it.
+		} else if (lua_gettop(Lua) - base != 1) {
+			DebugPrint("HandleCheats must return a boolean\n");
+		} else {
+			ret = LuaToBoolean(Lua, -1);
+		}
 	}
+	lua_settop(Lua, base);
 	return ret;
 }
 
@@ -783,7 +761,7 @@ static int InputKey(int key)
 #else
 			if (strncmp(Input, "ffw ", 4) == 0 && ReplayGameType != ReplayNone) {
 #endif
-				FastForwardCycle = atoi(&Input[4]);
+				FastForwardCycle = atol(&Input[4]);
 			}
 
 			if (Input[0]) {
@@ -932,7 +910,7 @@ int HandleKeyModifiersDown(unsigned key, unsigned keychar)
 		case SDLK_PRINT:
 			Screenshot();
 			if (GameRunning) {
-				SetMessage(_("Screenshot made."));
+				SetMessage("%s", _("Screenshot made."));
 			}
 			return 1;
 		default:
