@@ -127,7 +127,9 @@ void initGuichan()
 */
 void freeGuichan() 
 {
-	delete Gui->getGraphics();
+	if (Gui) {
+		delete Gui->getGraphics();
+	}
 	delete Gui;
 	delete Input;
 
@@ -1218,16 +1220,11 @@ void Windows::setBaseColor(const gcn::Color &color)
 */
 void LuaListModel::setList(lua_State *lua, lua_Object *lo)
 {
-	int args;
-	int j;
+	clear();
 
-	list.clear();
-
-	args = lua_objlen(lua, *lo);
-	for (j = 0; j < args; ++j) {
-		lua_rawgeti(lua, *lo, j + 1);
-		list.push_back(std::string(LuaToString(lua, -1)));
-		lua_pop(lua, 1);
+	int args = lua_objlen(lua, *lo);
+	for (int j = 0; j < args; ++j) {
+		add(std::string(LuaToString(lua, *lo, j + 1)));
 	}
 }
 
@@ -1340,6 +1337,14 @@ void ListBoxWidget::addActionListener(gcn::ActionListener *actionListener)
 	listbox.addActionListener(actionListener);
 }
 
+/**
+**  Get dirty
+*/
+bool ListBoxWidget::getDirty() const
+{
+	return listbox.getDirty();
+}
+
 
 /*----------------------------------------------------------------------------
 --  DropDownWidget
@@ -1394,13 +1399,9 @@ StatBoxWidget::StatBoxWidget(int width, int height) : percent(100)
 */
 void StatBoxWidget::draw(gcn::Graphics *graphics)
 {
-	int x;
-	int y;
 	int width;
 	int height;
 
-	x = getX();
-	y = getY();
 	width = getWidth();
 	height = getHeight();
 
