@@ -58,7 +58,7 @@
 ----------------------------------------------------------------------------*/
 
 // FIXME: make configurable
-#define MAGIC_FOR_NEW_UNITS  33         /// How many percent of max mana for new units
+#define CHARGE_FOR_NEW_UNITS  33         /// How many percent of max charge for new units
 
 /*----------------------------------------------------------------------------
 --  Variables
@@ -90,24 +90,62 @@ static std::vector<LabelsLaterStruct> LabelsLater;
 extern int GetSpriteIndex(const std::string &spriteName);
 
 /**
-**  Get the resource ID from a SCM object.
+**  Parse a "terrain" build restriction.
 **
-**  @param l  Lua state.
+**  @param l  Lua state where the value at the top of the stack
+**            is the table containing arguments for the "terrain"
+**            build restriction.  On exit, it will be the same.
 **
-**  @return   the resource id
+**  @return   The freshly constructed build restriction.
+**            The caller is responsible of deleting it.
+**
+**  If the argument table is invalid, this function indirectly calls
+**  lua_error and thus longjmp.  This function doesn't itself leak
+**  memory in that case, but its careless callers can do so.
 */
-unsigned CclGetResourceByName(lua_State *l)
+static CBuildRestrictionTerrain *ParseBuildRestrictionTerrain(lua_State *l)
 {
 	const char *value;
+	unsigned fieldFlags = 0;
+	int min = 0;
+	int max = INT_MAX;
 
-	value = LuaToString(l, -1);
-	for (int i = 0; i < MaxCosts; ++i) {
-		if (value == DefaultResourceNames[i]) {
-			return i;
+	for (lua_pushnil(l); lua_next(l, -2); lua_pop(l, 1)) {
+		value = LuaToString(l, -2);
+		if (!strcmp(value, "CountLand")) {
+			if (LuaToBoolean(l, -1)) {
+				fieldFlags |= MapFieldLandAllowed;
+			} else {
+				fieldFlags &= ~MapFieldLandAllowed;
+			}
+		} else if (!strcmp(value, "CountCoast")) {
+			if (LuaToBoolean(l, -1)) {
+				fieldFlags |= MapFieldCoastAllowed;
+			} else {
+				fieldFlags &= ~MapFieldCoastAllowed;
+			}
+		} else if (!strcmp(value, "CountShallowWater")) {
+			if (LuaToBoolean(l, -1)) {
+				fieldFlags |= MapFieldShallowWater;
+			} else {
+				fieldFlags &= ~MapFieldShallowWater;
+			}
+		} else if (!strcmp(value, "CountDeepWater")) {
+			if (LuaToBoolean(l, -1)) {
+				fieldFlags |= MapFieldDeepWater;
+			} else {
+				fieldFlags &= ~MapFieldDeepWater;
+			}
+		} else if (!strcmp(value, "Min")) {
+			min = LuaToNumber(l, -1);
+		} else if (!strcmp(value, "Max")) {
+			max = LuaToNumber(l, -1);
+		} else {
+			LuaError(l, "Unsupported BuildingRules terrain tag: %s" _C_ value);
 		}
 	}
-	LuaError(l, "Unsupported resource tag: %s" _C_ value);
-	return 0xABCDEF;
+
+	return new CBuildRestrictionTerrain(fieldFlags, min, max);
 }
 
 /**
@@ -126,14 +164,10 @@ static void ParseBuildingRules(lua_State *l, std::vector<CBuildRestriction *> &b
 	Assert(!(args & 1)); // must be even
 
 	for (int i = 0; i < args; ++i) {
-		lua_rawgeti(l, -1, i + 1);
-		value = LuaToString(l, -1);
-		lua_pop(l, 1);
+		value = LuaToString(l, -1, i + 1);
 		++i;
 		lua_rawgeti(l, -1, i + 1);
-		if (!lua_istable(l, -1)) {
-			LuaError(l, "incorrect argument");
-		}
+		LuaCheckTable(l, -1);
 		if (!strcmp(value, "distance")) {
 			CBuildRestrictionDistance *b = new CBuildRestrictionDistance;
 
@@ -201,6 +235,8 @@ static void ParseBuildingRules(lua_State *l, std::vector<CBuildRestriction *> &b
 				}
 			}
 			andlist->push_back(b);
+		} else if (!strcmp(value, "terrain")) {
+			andlist->push_back(ParseBuildRestrictionTerrain(l));
 		} else {
 			LuaError(l, "Unsupported BuildingRules tag: %s" _C_ value);
 		}
@@ -209,6 +245,22 @@ static void ParseBuildingRules(lua_State *l, std::vector<CBuildRestriction *> &b
 	blist.push_back(andlist);
 }
 
+/**
+**  Explicitly allow or disallow the unit type in the specified terrain.
+**  This overrides the automatic CUnitType::MovementMask computation.
+**  For use when the author of the Lua script knows best.
+*/
+static void SetExplicitTerrain(CUnitType *type, unsigned mask, bool allow)
+{
+	if (allow) {
+		type->ExplicitAllowTerrainMask |= mask;
+		type->ExplicitForbidTerrainMask &= ~mask;
+	} else {
+		type->ExplicitAllowTerrainMask &= ~mask;
+		type->ExplicitForbidTerrainMask |= mask;
+	}
+}
+		
 /**
 **  Parse unit-type.
 **
@@ -220,26 +272,22 @@ static int CclDefineUnitType(lua_State *l)
 	CUnitType *type;
 	const char *str;
 	int i;
-	int redefine;
 	int subargs;
 	int k;
 
 	LuaCheckArgs(l, 2);
-	if (!lua_istable(l, 2)) {
-		LuaError(l, "incorrect argument");
-	}
+	LuaCheckTable(l, 2);
 
 	// Slot identifier
 	str = LuaToString(l, 1);
 	type = UnitTypeByIdent(str);
-	if (type) {
-		redefine = 1;
-	} else {
+	if (!type) {
 		type = NewUnitTypeSlot(str);
-		redefine = 0;
 	}
 
 	type->NumDirections = 0;
+	type->ExplicitAllowTerrainMask = 0;
+	type->ExplicitForbidTerrainMask = 0;
 
 	//
 	//  Parse the list: (still everything could be changed!)
@@ -249,89 +297,53 @@ static int CclDefineUnitType(lua_State *l)
 		if (!strcmp(value, "Name")) {
 			type->Name = LuaToString(l, -1);
 		} else if (!strcmp(value, "Image")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
+			LuaCheckTable(l, -1);
 			subargs = lua_objlen(l, -1);
 			for (k = 0; k < subargs; ++k) {
-				lua_rawgeti(l, -1, k + 1);
-				value = LuaToString(l, -1);
-				lua_pop(l, 1);
+				value = LuaToString(l, -1, k + 1);
 				++k;
 
 				if (!strcmp(value, "file")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->File = LuaToString(l, -1);
-					lua_pop(l, 1);
+					type->File = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "size")) {
 					lua_rawgeti(l, -1, k + 1);
-					if (!lua_istable(l, -1)) {
-						LuaError(l, "incorrect argument");
-					}
-					lua_rawgeti(l, -1, 1);
-					type->Width = LuaToNumber(l, -1);
-					lua_pop(l, 1);
-					lua_rawgeti(l, -1, 2);
-					type->Height = LuaToNumber(l, -1);
-					lua_pop(l, 1);
+					LuaCheckTable(l, -1);
+					type->Width = LuaToNumber(l, -1, 1);
+					type->Height = LuaToNumber(l, -1, 2);
 					lua_pop(l, 1);
 				} else {
 					LuaError(l, "Unsupported image tag: %s" _C_ value);
 				}
 			}
 		} else if (!strcmp(value, "Shadow")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
+			LuaCheckTable(l, -1);
 			subargs = lua_objlen(l, -1);
 			for (k = 0; k < subargs; ++k) {
-				lua_rawgeti(l, -1, k + 1);
-				value = LuaToString(l, -1);
-				lua_pop(l, 1);
+				value = LuaToString(l, -1, k + 1);
 				++k;
 
 				if (!strcmp(value, "file")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->ShadowFile = LuaToString(l, -1);
-					lua_pop(l, 1);
+					type->ShadowFile = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "size")) {
 					lua_rawgeti(l, -1, k + 1);
-					if (!lua_istable(l, -1)) {
-						LuaError(l, "incorrect argument");
-					}
-					lua_rawgeti(l, -1, 1);
-					type->ShadowWidth = LuaToNumber(l, -1);
-					lua_pop(l, 1);
-					lua_rawgeti(l, -1, 2);
-					type->ShadowHeight = LuaToNumber(l, -1);
-					lua_pop(l, 1);
+					LuaCheckTable(l, -1);
+					type->ShadowWidth = LuaToNumber(l, -1, 1);
+					type->ShadowHeight = LuaToNumber(l, -1, 2);
 					lua_pop(l, 1);
 				} else if (!strcmp(value, "offset")) {
 					lua_rawgeti(l, -1, k + 1);
-					if (!lua_istable(l, -1)) {
-						LuaError(l, "incorrect argument");
-					}
-					lua_rawgeti(l, -1, 1);
-					type->ShadowOffsetX = LuaToNumber(l, -1);
-					lua_pop(l, 1);
-					lua_rawgeti(l, -1, 2);
-					type->ShadowOffsetY = LuaToNumber(l, -1);
-					lua_pop(l, 1);
+					LuaCheckTable(l, -1);
+					type->ShadowOffsetX = LuaToNumber(l, -1, 1);
+					type->ShadowOffsetY = LuaToNumber(l, -1, 2);
 					lua_pop(l, 1);
 				} else {
 					LuaError(l, "Unsupported shadow tag: %s" _C_ value);
 				}
 			}
 		} else if (!strcmp(value, "Offset")) {
-			if (!lua_istable(l, -1) || lua_objlen(l, -1) != 2) {
-				LuaError(l, "incorrect argument");
-			}
-			lua_rawgeti(l, -1, 1);
-			type->OffsetX = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 2);
-			type->OffsetY = LuaToNumber(l, -1);
-			lua_pop(l, 1);
+			LuaCheckTableSize(l, -1, 2);
+			type->OffsetX = LuaToNumber(l, -1, 1);
+			type->OffsetY = LuaToNumber(l, -1, 2);
 		} else if (!strcmp(value, "Flip")) {
 			type->Flip = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "Animations")) {
@@ -395,46 +407,26 @@ static int CclDefineUnitType(lua_State *l)
 			type->BurnPercent = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "BurnDamageRate")) {
 			type->BurnDamageRate = LuaToNumber(l, -1);
-		} else if (!strcmp(value, "MaxMana")) {
-			type->Variable[MANA_INDEX].Max = LuaToNumber(l, -1);
-			type->Variable[MANA_INDEX].Value = (type->Variable[MANA_INDEX].Max * MAGIC_FOR_NEW_UNITS) / 100;
-			type->Variable[MANA_INDEX].Increase = 1;
-			type->Variable[MANA_INDEX].Enable = 1;
+		} else if (!strcmp(value, "MaxCharge")) {
+			type->Variable[CHARGE_INDEX].Max = LuaToNumber(l, -1);
+			type->Variable[CHARGE_INDEX].Value = (type->Variable[CHARGE_INDEX].Max * CHARGE_FOR_NEW_UNITS) / 100;
+			type->Variable[CHARGE_INDEX].Increase = 1;
+			type->Variable[CHARGE_INDEX].Enable = 1;
 		} else if (!strcmp(value, "TileSize")) {
-			if (!lua_istable(l, -1) || lua_objlen(l, -1) != 2) {
-				LuaError(l, "incorrect argument");
-			}
-			lua_rawgeti(l, -1, 1);
-			type->TileWidth = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 2);
-			type->TileHeight = LuaToNumber(l, -1);
-			lua_pop(l, 1);
+			LuaCheckTableSize(l, -1, 2);
+			type->TileWidth = LuaToNumber(l, -1, 1);
+			type->TileHeight = LuaToNumber(l, -1, 2);
 		} else if (!strcmp(value, "Decoration")) {
 			type->Decoration = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "NeutralMinimapColor")) {
-			if (!lua_istable(l, -1) || lua_objlen(l, -1) != 3) {
-				LuaError(l, "incorrect argument");
-			}
-			lua_rawgeti(l, -1, 1);
-			type->NeutralMinimapColorRGB.r = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 2);
-			type->NeutralMinimapColorRGB.g = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 3);
-			type->NeutralMinimapColorRGB.b = LuaToNumber(l, -1);
-			lua_pop(l, 1);
+			LuaCheckTableSize(l, -1, 3);
+			type->NeutralMinimapColorRGB.r = LuaToNumber(l, -1, 1);
+			type->NeutralMinimapColorRGB.g = LuaToNumber(l, -1, 2);
+			type->NeutralMinimapColorRGB.b = LuaToNumber(l, -1, 3);
 		} else if (!strcmp(value, "BoxSize")) {
-			if (!lua_istable(l, -1) || lua_objlen(l, -1) != 2) {
-				LuaError(l, "incorrect argument");
-			}
-			lua_rawgeti(l, -1, 1);
-			type->BoxWidth = LuaToNumber(l, -1);
-			lua_pop(l, 1);
-			lua_rawgeti(l, -1, 2);
-			type->BoxHeight = LuaToNumber(l, -1);
-			lua_pop(l, 1);
+			LuaCheckTableSize(l, -1, 2);
+			type->BoxWidth = LuaToNumber(l, -1, 1);
+			type->BoxHeight = LuaToNumber(l, -1, 2);
 		} else if (!strcmp(value, "NumDirections")) {
 			type->NumDirections = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "Revealer")) {
@@ -480,6 +472,19 @@ static int CclDefineUnitType(lua_State *l)
 				LuaError(l, "Unsupported Type: %s" _C_ value);
 			}
 
+		} else if (!strcmp(value, "AllowTerrainLand")) {
+			SetExplicitTerrain(type, MapFieldLandAllowed,
+					   LuaToBoolean(l, -1));
+		} else if (!strcmp(value, "AllowTerrainCoast")) {
+			SetExplicitTerrain(type, MapFieldCoastAllowed,
+					   LuaToBoolean(l, -1));
+		} else if (!strcmp(value, "AllowTerrainShallowWater")) {
+			SetExplicitTerrain(type, MapFieldShallowWater,
+					   LuaToBoolean(l, -1));
+		} else if (!strcmp(value, "AllowTerrainDeepWater")) {
+			SetExplicitTerrain(type, MapFieldDeepWater,
+					   LuaToBoolean(l, -1));
+
 		} else if (!strcmp(value, "RightMouseAction")) {
 			value = LuaToString(l, -1);
 			if (!strcmp(value, "none")) {
@@ -492,8 +497,6 @@ static int CclDefineUnitType(lua_State *l)
 				type->MouseAction = MouseActionHarvest;
 			} else if (!strcmp(value, "spell-cast")) {
 				type->MouseAction = MouseActionSpellCast;
-			} else if (!strcmp(value, "sail")) {
-				type->MouseAction = MouseActionSail;
 			} else {
 				LuaError(l, "Unsupported RightMouseAction: %s" _C_ value);
 			}
@@ -530,9 +533,7 @@ static int CclDefineUnitType(lua_State *l)
 		} else if (!strcmp(value, "VisibleUnderFog")) {
 			type->VisibleUnderFog = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "BuildingRules")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
+			LuaCheckTable(l, -1);
 			subargs = lua_objlen(l, -1);
 			// Free any old restrictions if they are redefined
 			for (std::vector<CBuildRestriction *>::iterator b = type->BuildingRules.begin();
@@ -542,20 +543,12 @@ static int CclDefineUnitType(lua_State *l)
 			type->BuildingRules.clear();
 			for (k = 0; k < subargs; ++k) {
 				lua_rawgeti(l, -1, k + 1);
-				if (!lua_istable(l, -1)) {
-					LuaError(l, "incorrect argument");
-				}
+				LuaCheckTable(l, -1);
 				ParseBuildingRules(l, type->BuildingRules);
 				lua_pop(l, 1);
 			}
 		} else if (!strcmp(value, "ShoreBuilding")) {
 			type->ShoreBuilding = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "LandUnit")) {
-			type->LandUnit = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "AirUnit")) {
-			type->AirUnit = LuaToBoolean(l, -1);
-		} else if (!strcmp(value, "SeaUnit")) {
-			type->SeaUnit = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "Indestructible")) {
 			type->Indestructible = LuaToNumber(l, -1);
 		} else if (!strcmp(value, "CanTransport")) {
@@ -573,9 +566,7 @@ static int CclDefineUnitType(lua_State *l)
 		} else if (!strcmp(value, "Vanishes")) {
 			type->Vanishes = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "CanCastSpell")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
+			LuaCheckTable(l, -1);
 			//
 			// Warning: can-cast-spell should only be used AFTER all spells
 			// have been defined. FIXME: MaxSpellType=500 or something?
@@ -593,19 +584,15 @@ static int CclDefineUnitType(lua_State *l)
 			for (k = 0; k < subargs; ++k) {
 				const SpellType *spell;
 
-				lua_rawgeti(l, -1, k + 1);
-				value = LuaToString(l, -1);
+				value = LuaToString(l, -1, k + 1);
 				spell = SpellTypeByIdent(value);
 				if (spell == NULL) {
 					LuaError(l, "Unknown spell type: %s" _C_ value);
 				}
-				lua_pop(l, 1);
 				type->CanCastSpell[spell->Slot] = 1;
 			}
 		} else if (!strcmp(value, "AutoCastActive")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
+			LuaCheckTable(l, -1);
 			//
 			// Warning: AutoCastActive should only be used AFTER all spells
 			// have been defined.
@@ -623,8 +610,7 @@ static int CclDefineUnitType(lua_State *l)
 			for (k = 0; k < subargs; ++k) {
 				const SpellType *spell;
 
-				lua_rawgeti(l, -1, k + 1);
-				value = LuaToString(l, -1);
+				value = LuaToString(l, -1, k + 1);
 				spell = SpellTypeByIdent(value);
 				if (spell == NULL) {
 					LuaError(l, "AutoCastActive : Unknown spell type: %s" _C_ value);
@@ -632,7 +618,6 @@ static int CclDefineUnitType(lua_State *l)
 				if (!spell->AutoCast) {
 					LuaError(l, "AutoCastActive : Define autocast method for %s." _C_ value);
 				}
-				lua_pop(l, 1);
 				type->AutoCastActive[spell->Slot] = 1;
 			}
 		} else if (!strcmp(value, "ProductionEfficiency")) {
@@ -644,44 +629,22 @@ static int CclDefineUnitType(lua_State *l)
 		} else if (!strcmp(value, "Organic")) {
 			type->Organic = LuaToBoolean(l, -1);
 		} else if (!strcmp(value, "Sounds")) {
-			if (!lua_istable(l, -1)) {
-				LuaError(l, "incorrect argument");
-			}
+			LuaCheckTable(l, -1);
 			subargs = lua_objlen(l, -1);
 			for (k = 0; k < subargs; ++k) {
-				lua_rawgeti(l, -1, k + 1);
-				value = LuaToString(l, -1);
-				lua_pop(l, 1);
+				value = LuaToString(l, -1, k + 1);
 				++k;
 
 				if (!strcmp(value, "selected")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Selected.Name = LuaToString(l, -1);
-					lua_pop(l, 1);
+					type->Sound.Selected.Name = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "acknowledge")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Acknowledgement.Name = LuaToString(l, -1);
-					lua_pop(l, 1);
+					type->Sound.Acknowledgement.Name = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "ready")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Ready.Name = LuaToString(l, -1);
-					lua_pop(l, 1);
-				} else if (!strcmp(value, "repair")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Repair.Name = LuaToString(l, -1);
-					lua_pop(l, 1);
-				} else if (!strcmp(value, "harvest")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Harvest.Name = LuaToString(l, -1);
-					lua_pop(l, 1);
+					type->Sound.Ready.Name = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "help")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Help.Name = LuaToString(l, -1);
-					lua_pop(l, 1);
+					type->Sound.Help.Name = LuaToString(l, -1, k + 1);
 				} else if (!strcmp(value, "dead")) {
-					lua_rawgeti(l, -1, k + 1);
-					type->Sound.Dead.Name = LuaToString(l, -1);
-					lua_pop(l, 1);
+					type->Sound.Dead.Name = LuaToString(l, -1, k + 1);
 				} else {
 					LuaError(l, "Unsupported sound tag: %s" _C_ value);
 				}
@@ -723,6 +686,7 @@ static int CclDefineUnitType(lua_State *l)
 	return 0;
 }
 
+
 // ----------------------------------------------------------------------------
 
 /**
@@ -925,9 +889,7 @@ static CAnimation *ParseAnimation(lua_State *l, int idx)
 	int j;
 	const char *str;
 
-	if (!lua_istable(l, idx)) {
-		LuaError(l, "incorrect argument");
-	}
+	LuaCheckTable(l, idx);
 	args = lua_objlen(l, idx);
 	anim = new CAnimation[args + 1];
 	tail = NULL;
@@ -935,9 +897,7 @@ static CAnimation *ParseAnimation(lua_State *l, int idx)
 	LabelsLater.clear();
 
 	for (j = 0; j < args; ++j) {
-		lua_rawgeti(l, idx, j + 1);
-		str = LuaToString(l, -1);
-		lua_pop(l, 1);
+		str = LuaToString(l, idx, j + 1);
 		ParseAnimationFrame(l, str, &anim[j]);
 		if (!tail) {
 			tail = &anim[j];
@@ -976,9 +936,7 @@ static int CclDefineAnimations(lua_State *l)
 	CAnimations *anims;
 
 	LuaCheckArgs(l, 2);
-	if (!lua_istable(l, 2)) {
-		LuaError(l, "incorrect argument");
-	}
+	LuaCheckTable(l, 2);
 
 	name = LuaToString(l, 1);
 	anims = AnimationsByIdent(name);
@@ -1162,19 +1120,13 @@ static int CclDefineDecorations(lua_State *l)
 				tmp.Index = GetVariableIndex(LuaToString(l, -1));
 				Assert(tmp.Index != -1);
 			} else if (!strcmp(key, "Offset")) {
-				Assert(lua_istable(l, -1));
-				lua_rawgeti(l, -1, 1); // X
-				lua_rawgeti(l, -2, 2); // Y
-				tmp.OffsetX = LuaToNumber(l, -2);
-				tmp.OffsetY = LuaToNumber(l, -1);
-				lua_pop(l, 2); // Pop X and Y
+				LuaCheckTableSize(l, -1, 2);
+				tmp.OffsetX = LuaToNumber(l, -1, 1);
+				tmp.OffsetY = LuaToNumber(l, -1, 2);
 			} else if (!strcmp(key, "OffsetPercent")) {
-				Assert(lua_istable(l, -1));
-				lua_rawgeti(l, -1, 1); // X
-				lua_rawgeti(l, -2, 2); // Y
-				tmp.OffsetXPercent = LuaToNumber(l, -2);
-				tmp.OffsetYPercent = LuaToNumber(l, -1);
-				lua_pop(l, 2); // Pop X and Y
+				LuaCheckTableSize(l, -1, 2);
+				tmp.OffsetXPercent = LuaToNumber(l, -1, 1);
+				tmp.OffsetYPercent = LuaToNumber(l, -1, 2);
 			} else if (!strcmp(key, "CenterX")) {
 				tmp.IsCenteredInX = LuaToBoolean(l, -1);
 			} else if (!strcmp(key, "CenterY")) {
@@ -1195,7 +1147,7 @@ static int CclDefineDecorations(lua_State *l)
 				Assert(lua_istable(l, -1));
 				lua_rawgeti(l, -1, 1); // MethodName
 				lua_rawgeti(l, -2, 2); // Data
-				Assert(lua_istable(l, -1));
+				LuaCheckTable(l, -1);
 				key = LuaToString(l, -2);
 				if (!strcmp(key, "sprite")) {
 					CDecoVarSpriteBar *decovarspritebar = new CDecoVarSpriteBar;
@@ -1250,7 +1202,7 @@ void UpdateUnitVariables(const CUnit *unit)
 	type = unit->Type;
 	for (int i = 0; i < NVARALREADYDEFINED; i++) { // default values
 		if (i == ARMOR_INDEX || i == PIERCINGDAMAGE_INDEX ||
-				i == BASICDAMAGE_INDEX || i == MANA_INDEX ||
+				i == BASICDAMAGE_INDEX || i == CHARGE_INDEX ||
 				i == KILL_INDEX || i == HP_INDEX) {
 			continue;
 		}
@@ -1345,7 +1297,7 @@ void UpdateUnitVariables(const CUnit *unit)
 */
 void InitDefinedVariables()
 {
-	const char *var[NVARALREADYDEFINED] = {"HitPoints", "Build", "Mana", "Transport",
+	const char *var[NVARALREADYDEFINED] = {"HitPoints", "Build", "Charge", "Transport",
 		"Training", "GiveResource", "Kill", "Armor", "SightRange",
 		"AttackRange", "PiercingDamage", "BasicDamage", "PosX", "PosY", "RadarRange",
 		"RadarJammerRange", "AutoRepairRange", "Slot"};
