@@ -59,7 +59,7 @@
 ----------------------------------------------------------------------------*/
 
 /**
-**  Decoration: health, mana.
+**  Decoration: health, charge.
 */
 class Decoration {
 public:
@@ -229,25 +229,15 @@ static int CclDefineSprites(lua_State *l)
 			} else if (!strcmp(key, "File")) {
 				deco.File = LuaToString(l, -1);
 			} else if (!strcmp(key, "Offset")) {
-				if (!lua_istable(l, -1) || lua_objlen(l, -1) != 2) {
-					LuaError(l, "incorrect argument");
-				}
-				lua_rawgeti(l, -1, 1); // offsetX
-				lua_rawgeti(l, -2, 2); // offsetY
-				deco.HotX = LuaToNumber(l, -2);
-				deco.HotY = LuaToNumber(l, -1);
-				lua_pop(l, 2); // Pop offsetX and Y
+				LuaCheckTableSize(l, -1, 2);
+				deco.HotX = LuaToNumber(l, -1, 1);
+				deco.HotY = LuaToNumber(l, -1, 2);
 			} else if (!strcmp(key, "Size")) {
-				if (!lua_istable(l, -1) || lua_objlen(l, -1) != 2) {
-					LuaError(l, "incorrect argument");
-				}
-				lua_rawgeti(l, -1, 1); // Width
-				lua_rawgeti(l, -2, 2); // Height
-				deco.Width = LuaToNumber(l, -2);
-				deco.Height = LuaToNumber(l, -1);
-				lua_pop(l, 2); // Pop Width and Height
+				LuaCheckTableSize(l, -1, 2);
+				deco.Width = LuaToNumber(l, -1, 1);
+				deco.Height = LuaToNumber(l, -1, 2);
 			} else { // Error.
-				LuaError(l, "incorrect field '%s' for the DefineSprite." _C_ key);
+				LuaError(l, "incorrect field '%s' for DefineSprites." _C_ key);
 			}
 			lua_pop(l, 1); // pop the value;
 		}
@@ -619,6 +609,10 @@ void ShowOrder(const CUnit *unit)
 	if (unit->Destroyed) {
 		return;
 	}
+	
+	if (unit->Player != ThisPlayer && !ThisPlayer->IsAllied(unit)) {
+        	return;
+	}
 
 	// Get current position
 	x1 = CurrentViewport->Map2ViewportX(
