@@ -1090,11 +1090,11 @@ CFont *CFont::New(const std::string &ident, CGraphic *g)
 		swap(fontFamily->G, gptr);
 		return &fontFamily->MarkupFont;
 	} else {
-		std::auto_ptr<CFontFamily> fontFamily(new CFontFamily(ident));
+		CFontFamily *fontFamily = new CFontFamily(ident);
 		swap(fontFamily->G, gptr);
 		FontFamilies.insert(
-			FontFamiliesType::value_type(ident, fontFamily.get()));
-		return &fontFamily.release()->MarkupFont;
+			FontFamiliesType::value_type(ident, fontFamily));
+		return &fontFamily->MarkupFont;
 	}
 }
 
