@@ -1,9 +1,9 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 # -*- coding: utf-8 -*-
 #
 #  Build script for the Bos Wars engine.
 #
-#  (c) Copyright 2010-2013 by Francois Beerten.
+#  (c) Copyright 2010-2023 by Francois Beerten.
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 import os
 import glob
 import sys
+import shutil
 try:
    import Queue as queue
 except ImportError:
@@ -41,6 +42,8 @@ def find(startdir, pattern):
     import fnmatch
     results = []
     for dirpath,dirnames,files in os.walk(startdir):
+       dirnames.sort()
+       files.sort()
        for f in files:
           # Ignore dot files.  GNU Emacs especially creates lock files
           # with names like ".#script.cpp".  Don't try to compile those.
@@ -50,11 +53,17 @@ def find(startdir, pattern):
     
 sources = find('engine', '*.cpp')
 
+def tolist(x):
+    if isinstance(x, str):
+        return x.split(' ')
+    else:
+        return list(x)
+
 class Gcc(object):
   def __init__(self, cflags=[], ldflags=[], cc='g++', builddir='fbuild', 
                       usepkgconfig=True):
-     self.cflags = gccflags + list(cflags)
-     self.ldflags = list(ldflags)
+     self.cflags = gccflags + tolist(cflags)
+     self.ldflags = tolist(ldflags)
      self.cc = cc
      self.builddir = builddir
      self.usepkgconfig = usepkgconfig
@@ -84,7 +93,7 @@ class Gcc(object):
   def cxx(self, target, source, flags=[]):
      cflags = flags + self.cflags
      target = self.oname(target)
-     return (self.cc, '-c', source, '-o', self.oname(target), cflags)
+     return (self.cc, '-c', source, '-o', target, cflags)
   def ld(self, target, sources, flags=[]):
      ldflags = flags + self.ldflags
      objects = [self.oname(s) for s in sources]
@@ -238,10 +247,12 @@ def pkgconfig(b, package):
         b.ldflags += cmdline2list(shell('pkg-config', '--libs', package).decode())
     except ExecutionError as e:
         return False
+    except OSError as e:
+        return False
     return True
 
-def CheckLib(b, lib, header=''):
-    if Check(b, lib, header):
+def CheckLib(b, lib, header='', function=''):
+    if Check(b, lib, header, function=function):
        b.lib(lib)
        return True
     return False
@@ -315,7 +326,7 @@ def detectEmbedable(b):
     RequireLib(b, 'png', 'png.h')
     if CheckLib(b, 'vorbis'):
        b.define('USE_VORBIS')
-    if CheckLib(b, 'theora'):
+    if CheckLib(b, 'theora', function='theora_decode_packetin'):
        b.define('USE_THEORA')
     if CheckLib(b, 'ogg'):
        b.define('USE_OGG')
@@ -462,7 +473,7 @@ def clean():
 def pot():
     luas = find('.', '*.lua')
     luas.sort()
-    run('xgettext','-d','bos','-k_','-o','languages/bos.pot', luas)
+    run('xgettext','-d','bos','--from-code', 'utf-8', '-k_','-o','languages/bos.pot', luas)
     s = sorted(sources)
     run('xgettext','-d','engine','-C','-k_','--add-comments=TRANSLATORS','-o','languages/engine.pot', s)
 
@@ -473,6 +484,46 @@ def all(**kwargs):
 def default(**kwargs):
     release(**kwargs)
 
+
+def copyfiles(files, destdir):
+    for f in files:
+        dirpath = destdir + '/' + os.path.dirname(f)
+        os.makedirs(dirpath, exist_ok=True)
+        shutil.copy2(f, dirpath)
+
+def install_data(datadir='./dist'):
+    copyfiles(glob.glob('*.txt'), datadir)
+    copyfiles(glob.glob('CHANGELOG*'), datadir)
+    copyfiles(glob.glob('campaigns/*/*.*'), datadir)
+    copyfiles(glob.glob('campaigns/*/*/*.*'), datadir)
+    copyfiles(glob.glob('doc/*.txt'), datadir)
+    copyfiles(glob.glob('doc/*.html'), datadir)
+    copyfiles(glob.glob('graphics/*/*.png'), datadir)
+    copyfiles(glob.glob('graphics/*/*/*.png'), datadir)
+    copyfiles(glob.glob('intro/*.*'), datadir)
+    copyfiles(glob.glob('languages/*.po'), datadir)
+    copyfiles(glob.glob('maps/*.map/*.*'), datadir)
+    copyfiles(glob.glob('maps/campaigns/*/*.map/*.*'), datadir)
+    copyfiles(glob.glob('patches/*.lua'), datadir)
+    copyfiles(glob.glob('patches/*.png'), datadir)
+    copyfiles(glob.glob('patches/*/*.png'), datadir)
+    copyfiles(glob.glob('scripts/*.lua'), datadir)
+    copyfiles(glob.glob('scripts/*/*.lua'), datadir)
+    copyfiles(glob.glob('scripts/*/*/*.lua'), datadir)
+    copyfiles(glob.glob('intro/*.*'), datadir)
+    copyfiles(glob.glob('sounds/*.*'), datadir)
+    copyfiles(glob.glob('sounds/ui/*.*'), datadir)
+    copyfiles(glob.glob('units/*/*.*'), datadir)
+
+
+def install(installdir='./dist', datadir=None, bindir=None, builddir='fbuild/release/', **kwargs):
+    datadir = datadir or installdir
+    bindir = bindir or installdir
+    release(builddir=builddir, **kwargs)
+    install_data(installdatadir)
+    shutil.copy2(builddir+'/'+ 'boswars', installdir)
+
+
 setup(default='default')
 if __name__ == '__main__':
     import optparse
