@@ -27,7 +27,14 @@
 --
 
 
-print("Bos Wars default config file loading ...\n")
+if (CompileFeature("DEBUG")) then
+  DebugPrint = print
+else
+  function DebugPrint(x) end
+end
+
+
+DebugPrint("Bos Wars default config file loading ...\n")
 
 -------------------------------------------------------------------------------
 --	Config-Part
@@ -49,7 +56,7 @@ playlist = {}
 local musiclist = ListFilesInDirectory("music/")
 for i,f in ipairs(musiclist) do
   if(string.find(f, ".ogg$") or string.find(f, ".wav$") or string.find(f, ".mp3$")) then 
-    print("Added music file:" .. f) 
+    DebugPrint("Added music file:" .. f) 
     playlist[i] = f
   end
 end
@@ -59,8 +66,8 @@ Preference.ShowAttackRange = false
 Preference.ShowReactionRange = false
 Preference.ShowOrders = 2
 
--- Mana and Health sprites
-DefineSprites({Name = "sprite-mana", File = "general/mana2.png",
+-- Charge and Health sprites
+DefineSprites({Name = "sprite-charge", File = "general/mana2.png",
   Offset = {0, -1}, Size = {31, 4}})
 DefineSprites({Name = "sprite-health", File = "general/health2.png",
   Offset = {0, -4}, Size = {31, 4}})
@@ -70,15 +77,15 @@ DefineDecorations({Index = "HitPoints", HideNeutral = true, CenterX = true,
   OffsetPercent = {50, 100}, Method = {"sprite", {"sprite-health"}},
   ShowWhenMax = false})
 
--- ManaDot
-DefineDecorations({Index = "Mana", HideNeutral = true, CenterX = true,
-  OffsetPercent = {50, 100},Method = {"sprite", {"sprite-mana"}},
+-- ChargeDot
+DefineDecorations({Index = "Charge", HideNeutral = true, CenterX = true,
+  OffsetPercent = {50, 100},Method = {"sprite", {"sprite-charge"}},
   ShowWhenMax = false})
 DefineDecorations({Index = "Transport", HideNeutral = true, CenterX = true,
-  OffsetPercent = {50, 100},Method = {"sprite", {"sprite-mana"}},
+  OffsetPercent = {50, 100},Method = {"sprite", {"sprite-charge"}},
   ShowWhenMax = false})
 DefineDecorations({Index = "Training", HideNeutral = true, CenterX = true,
-  OffsetPercent = {50, 100},Method = {"sprite", {"sprite-mana"}},
+  OffsetPercent = {50, 100},Method = {"sprite", {"sprite-charge"}},
   ShowWhenMax = false})
 
 -------------------------------------------------------------------------------
@@ -117,6 +124,7 @@ UseGLTextureCompression = false
 -------------------------------------------------------------------------------
 
 DefineDefaultResourceNames("energy", "magma")
+DefineDefaultDisplayResourceNames(_("Energy"), _("Magma"))
 
 DefinePlayerColorIndex(208, 4)
 
@@ -133,8 +141,8 @@ DefinePlayerColors({
 })
 
 function InitGameVariables()
-   SetSpeeds(1)
-   InitAiScripts()
+  SetSpeeds(1)
+  ClearAiState()
 end
 
 --  Pathfinder
@@ -178,7 +186,7 @@ if (preferences == nil) then
     VideoWidth = 800,
     VideoHeight = 600,
     VideoFullScreen = false,
-    PlayerName = "Player",
+    PlayerName = nil,
     FogOfWar = true,
     ShowCommandKey = true,
     GroupKeys = "0123456789`",
@@ -202,15 +210,22 @@ if (preferences.MaxOpenGLTexture == nil) then
    preferences.MaxOpenGLTexture = 512
 end
 
+-- If PlayerName is not in the preferences, then use
+-- whatever default the engine has chosen.
+if preferences.PlayerName ~= nil then
+  SetLocalPlayerName(preferences.PlayerName)
+end
+
 UseOpenGL = preferences.UseOpenGL
 SetVideoResolution(preferences.VideoWidth, preferences.VideoHeight)
 SetVideoFullScreen(preferences.VideoFullScreen)
 SetMaxOpenGLTexture(preferences.MaxOpenGLTexture)
-SetLocalPlayerName(preferences.PlayerName)
 SetFogOfWar(preferences.FogOfWar)
 UI.ButtonPanel.ShowCommandKey = preferences.ShowCommandKey
 SetGroupKeys(preferences.GroupKeys)
-SetGameSpeed(preferences.GameSpeed)
+if GetGameSpeed() < 0 then
+  SetGameSpeed(preferences.GameSpeed)
+end
 SetEffectsEnabled(preferences.EffectsEnabled)
 SetEffectsVolume(preferences.EffectsVolume)
 SetMusicEnabled(preferences.MusicEnabled)
@@ -236,6 +251,6 @@ Load("scripts/cheats.lua")
 Load("scripts/patches.lua")
 
 
-default_objective = _("Eliminate your enemies.")
+default_objectives = {_("Eliminate your enemies.")}
 
-print("... ready!")
+DebugPrint("... ready!")
