@@ -9,7 +9,7 @@
 --
 --      guichan.lua - The main UI lua script.
 --
---      (c) Copyright 2005-2010 by François Beerten
+--      (c) Copyright 2005-2023 by François Beerten
 --
 --      This program is free software; you can redistribute it and/or modify
 --      it under the terms of the GNU General Public License as published by
@@ -238,18 +238,23 @@ function AddMenuHelpers(menu)
     -- Otherwise call the user's callback
     local function cb(s)
       local f = bq:getSelectedItem()
+
       if (f == "../") then
-        local i
-        for i = string.len(bq.path) - 1, 1, -1 do
-          if (string.sub(bq.path, i, i) == "/") then
-            bq.path = string.sub(bq.path, 1, i)
-            updateList()
-            break
+        if (s ~= "select") then
+          local i
+          for i = string.len(bq.path) - 1, 1, -1 do
+            if (string.sub(bq.path, i, i) == "/") then
+              bq.path = string.sub(bq.path, 1, i)
+              updateList()
+              break
+            end
           end
         end
       elseif (string.sub(f, string.len(f)) == '/') then
-        bq.path = bq.path .. f
-        updateList()
+        if (s ~= "select") then
+          bq.path = bq.path .. f
+          updateList()
+        end
       else
         if (bq.actioncb ~= nil) then
           bq:actioncb(s)
@@ -329,7 +334,7 @@ function AddMenuHelpers(menu)
   function menu:addTextInputField(text, x, y, w)
     local b = TextField(text)
     b:setActionCallback(function() end) --FIXME: remove this?
-    b:setFont(Fonts["game"])
+    b:setFont(Fonts["game"]:PlainText())
     b:setBaseColor(clear)
     b:setForegroundColor(clear)
     b:setBackgroundColor(dark)
@@ -429,13 +434,13 @@ function RunResultsMenu()
   menu:run()
 end
 
-function RunMap(map, objective, fow, revealmap)
-  if objective == nil then
-    SetObjectives(default_objective)
+function RunMap(map, objectives, fow, revealmap)
+  if (objectives == nil) then
+    SetObjectives(default_objectives)
   else
-    SetObjectives(objective)
+    SetObjectives(objectives)
   end
-  loop = true
+  local loop = true
   while (loop) do
     InitGameVariables()
     if fow ~= nil then
@@ -452,17 +457,13 @@ function RunMap(map, objective, fow, revealmap)
   RunResultsMenu()
 end
 
-current_objectives = ""
+current_objectives = {}
 function ClearObjectives()
-  current_objectives = ""
+  current_objectives = {}
 end
 
 function AddObjective(objective)
-  if (current_objectives ~= "") then
-    current_objectives = current_objective .. "\n" .. objective
-  else
-    current_objectives = objective
-  end
+  table.insert(current_objectives, objective)
 end
 
 function SetObjectives(objectives)
@@ -497,7 +498,7 @@ function GameStarting()
   Cheater = false
 
   -- FIXME: get the version from somewhere else
-  UI.StatusLine:Set("Bos Wars V2.6.1 (c) 1998-2010 by the Bos Wars Team.")
+  UI.StatusLine:Set("Bos Wars V2.8.0 (c) 1998-2023 by the Bos Wars Team.")
 end
 
 
@@ -578,13 +579,15 @@ function RunStartGameMenu(s)
 
   AllowAllUnits()
   local function startgamebutton(s)
-    print("Starting map -------")
+    preferences.FogOfWar = fow:isMarked()
+    SavePreferences()
+    DebugPrint("Starting map -------")
     RunMap(browser:getSelectedMap(), nil, fow:isMarked(),
            revealmap:isMarked())
     PresentMap = OldPresentMap
     menu:stop()
   end
-  menu:addButton(_("~!Main Menu"), Video.Width / 2 - 250, Video.Height - 100,
+  menu:addButton(_("Main Menu (~<Esc~>)"), Video.Width / 2 - 250, Video.Height - 100,
                  function() menu:stop() end)
   menu:addButton(_("~!Start"), Video.Width / 2 + 50 ,  Video.Height - 100,
                  startgamebutton)
@@ -608,7 +611,7 @@ function RunReplayMenu(s)
   local reveal = menu:addCheckBox(_("Reveal map"), 100, 250, function() end)
 
   function startreplaybutton(s)
-    print("Starting map -------")
+    DebugPrint("Starting map -------")
     ResetMapOptions()
     InitGameVariables()
     StartReplay("~logs/" .. browser:getSelectedItem(), reveal:isMarked())
@@ -616,7 +619,7 @@ function RunReplayMenu(s)
     menu:stop()
   end
 
-  menu:addButton(_("~!Main Menu"), Video.Width / 2 - 250, Video.Height - 100,
+  menu:addButton(_("Main Menu (~<Esc~>)"), Video.Width / 2 - 250, Video.Height - 100,
                  function() menu:stop() end)
   menu:addButton(_("~!Start"), Video.Width / 2 + 50 ,  Video.Height - 100,
                  startreplaybutton)
@@ -634,9 +637,9 @@ function RunLoadGameMenu(s)
   local browser = menu:addBrowser("~save", lister, 
                                  Video.Width / 2 - 150, 100, 300, 200)
   function startgamebutton(s)
-      print("Starting saved game")
+      DebugPrint("Starting saved game")
       currentCampaign = nil
-      loop = true
+      local loop = true
       while (loop) do
         InitGameVariables()
         SavedGame = true
@@ -660,7 +663,7 @@ function RunLoadGameMenu(s)
       end
     menu:stop()
   end
-  menu:addButton(_("~!Main Menu"), Video.Width / 2 - 250, Video.Height - 100,
+  menu:addButton(_("Main Menu (~<Esc~>)"), Video.Width / 2 - 250, Video.Height - 100,
                  function() menu:stop() end)
   menu:addButton(_("~!Start"), Video.Width / 2 + 50 ,  Video.Height - 100,
                  startgamebutton)
@@ -672,18 +675,38 @@ end
 function RunEditorMenu(s)
   local menu
   local x = Video.Width / 2 - 100
+  local x1 = Video.Width / 3 - 100
+  local x2 = Video.Width * 2 / 3 - 100
 
   menu = BosMenu(_("Editor"))
 
-  menu:addButton(_("Create ~!New Map"), x, 220,
-    function() RunEditorNewMenu(); menu:stop() end)
-  menu:addButton(_("~!Load Map"), x, 260,
-    function() RunEditorLoadMenu(); menu:stop() end)
+  menu:addButton(_("Create ~!New Map"), x1, 220,
+    function()
+      if RunEditorNewMenu() then
+        menu:stop()
+      end
+    end)
+  menu:addButton(_("~!Load Map"), x1, 260,
+    function()
+      if RunEditorLoadMenu() then
+        menu:stop()
+      end
+    end)
 
-  menu:addButton(_("~!Patch Editor"), x, 320,
-    function() RunPatchEditorMenu(); menu:stop() end)
+  menu:addButton(_("C~!reate New Patch"), x2, 220,
+    function()
+      if RunNewPatchMenu() then
+        menu:stop()
+      end
+    end)
+  menu:addButton(_("Load ~!Patch"), x2, 260,
+    function()
+      if RunLoadPatchMenu() then
+        menu:stop()
+      end
+    end)
 
-  menu:addButton(_("~!Cancel"), x, Video.Height - 100,
+  menu:addButton(_("Main Menu (~<Esc~>)"), x, Video.Height - 100,
     function() menu:stop() end)
 
   menu:run()
@@ -695,8 +718,9 @@ function RunEditorNewMenu()
   local xsize
   local ysize
   local defaultSize = {64, 64}
+  local returnToMainMenu = false
 
-  menu = BosMenu(_("Editor"))
+  menu = BosMenu(_("Create New Map"))
 
   function starteditorbutton(s)
     local n = tonumber(xsize:getText())
@@ -706,9 +730,11 @@ function RunEditorNewMenu()
     if (n == nil) then n = 64 end
     Map.Info.MapHeight = n
 
+    ClearAiState()
     StartEditor(nil)
     Load("scripts/uilayout.lua")
     HandleCommandKey = HandleIngameCommandKey
+    returnToMainMenu = true
     menu:stop()
   end
 
@@ -728,12 +754,13 @@ function RunEditorNewMenu()
   ysize = menu:addTextInputField(tostring(defaultSize[2]),
     Video.Width / 2 + 10 + 60, 8 * sy, 40)
 
-  menu:addButton(_("~!Main Menu"), Video.Width / 2 - 250, Video.Height - 100,
+  menu:addButton(_("Cancel (~<Esc~>)"), Video.Width / 2 - 250, Video.Height - 100,
     function() menu:stop() end)
   menu:addButton(_("Start ~!Editor"), Video.Width / 2 + 50, Video.Height - 100,
     starteditorbutton)
 
   menu:run()
+  return returnToMainMenu
 end
 
 function RunEditorLoadMenu()
@@ -742,8 +769,9 @@ function RunEditorLoadMenu()
   local sy = Video.Height / 20
   local selectedmap = "islandwar.map"
   local numplayers = 2
+  local returnToMainMenu = false
 
-  menu = BosMenu(_("Editor"))
+  menu = BosMenu(_("Select Map to Edit"))
 
   menu:writeLargeText(_("Map"), sx, sy*3)
   menu:writeText(_("File:"), sx, sy*3+30)
@@ -753,7 +781,8 @@ function RunEditorLoadMenu()
   menu:writeText(_("Size:"), sx, sy*3+70)
   local mapsize = menu:writeText("       ", sx+70, sy*3+70)
   menu:writeText(_("Description:"), sx, sy*3+90)
-  local descr = menu:writeText("                                        ", sx+20, sy*3+110)
+  local descr = menu:writeText("No map", sx+20, sy*3+110)
+  descr:setWidth(sx * 9 - 20 - 20)
 
   local OldPresentMap = PresentMap
   PresentMap = function(description, nplayers, w, h, id)
@@ -775,18 +804,21 @@ function RunEditorLoadMenu()
   browser:setActionCallback(selectMap)
 
   function starteditorbutton(s)
+    ClearAiState()
     StartEditor(browser:getSelectedMap())
     Load("scripts/uilayout.lua")
     HandleCommandKey = HandleIngameCommandKey
     menu:stop()
+    returnToMainMenu = true
   end
-  menu:addButton(_("~!Main Menu"), Video.Width / 2 - 250, Video.Height - 100,
+  menu:addButton(_("Cancel (~<Esc~>)"), Video.Width / 2 - 250, Video.Height - 100,
                  function() menu:stop() end)
   menu:addButton(_("Start ~!Editor"), Video.Width / 2 + 50, Video.Height -100,
                  starteditorbutton)
 
   menu:run()
   PresentMap = OldPresentMap
+  return returnToMainMenu
 end
 
 Load("scripts/menus/network.lua")
