@@ -1,16 +1,16 @@
-#! /usr/bin/env python
+#! /usr/bin/env python3
 
-##     ____                _       __               
+##     ____                _       __
 ##    / __ )____  _____   | |     / /___ ___________
 ##   / __  / __ \/ ___/   | | /| / / __ `/ ___/ ___/
-##  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  ) 
-## /_____/\____/____/     |__/|__/\__,_/_/  /____/  
-##                                              
+##  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  )
+## /_____/\____/____/     |__/|__/\__,_/_/  /____/
+##
 ##       A futuristic real-time strategy game.
 ##          This file is part of Bos Wars.
 ##
 ##      Script that generates all the distribution packages.
-##      (c) Copyright 2007 by Francois Beerten
+##      (c) Copyright 2007-2023 by Francois Beerten
 ##
 ##      Stratagus is free software; you can redistribute it and/or modify
 ##      it under the terms of the GNU General Public License as published
@@ -32,7 +32,7 @@ def printDot():
   sys.stdout.flush()
 
 def createTarArchive(archivename, namelist):
-  print "\nCreating %s.tar.gz" % archivename
+  print("\nCreating %s.tar.gz" % archivename)
   tar = tarfile.open(archivename + ".tar.gz", "w:gz")
   for name in namelist:
       tarinfo = tar.gettarinfo(name, archivename + "/" + name)
@@ -41,12 +41,12 @@ def createTarArchive(archivename, namelist):
       tarinfo.uname = "user"
       tarinfo.gname = "user"
       printDot()
-      tar.addfile(tarinfo, file(name))
+      tar.addfile(tarinfo, open(name, 'rb'))
   tar.close()
   print
 
 def createZipArchive(archivename, namelist):
-  print "\nCreating %s.zip" % archivename
+  print("\nCreating %s.zip" % archivename)
   z = zipfile.ZipFile(archivename + ".zip", "w", zipfile.ZIP_DEFLATED)
   for name in namelist:
       z.write(name, archivename + "/" + name)
@@ -54,9 +54,9 @@ def createZipArchive(archivename, namelist):
   z.close()
   print
 
-
-def isSvnWorkSpace():
-   return os.access('.svn', os.F_OK)
+def listGitFiles():
+   entries = os.popen('git ls-files').readlines()
+   return [i.strip() for i in entries]
 
 def listSvnFiles():
    entries = os.popen('svn ls -R').readlines()
@@ -68,10 +68,12 @@ def listSvnFiles():
    return files
 
 def listFilesInDirectories():
-   raise "Not implemented"
+   raise RuntimeError("Not implemented")
 
 def listFiles():
-   if isSvnWorkSpace():
+   if os.access('.git', os.F_OK):
+      return listGitFiles()
+   elif os.access('.svn', os.F_OK):
       return listSvnFiles()
    else:
       return listFilesInDirectories()
@@ -89,13 +91,14 @@ def buildVersionName():
 
 def filterDirectories(files):
    return [x for x in files if not x.endswith('/')]
+
 def filterSources(files):
-   buildfiles = ['SConstruct', 'bos.sln', 'make.py', 'fabricate.py']
+   buildfiles = ['bos.sln', 'make.py', 'fabricate.py']
    def isGoodFile(f):
      return (not f.startswith('engine/') and not f.startswith('tools/') and
              f not in buildfiles)
    return [x for x in files if isGoodFile(x)]
-   
+
 
 
 def buildPackages():
@@ -110,7 +113,7 @@ def buildPackages():
   createZipArchive(name % 'src', files)
 
   # create binary packages
-  #os.popen('scons static=1 debug=0 CXX=apg++')
+  #  FIXME: automatically build a "static" boswars executable ?
   binarydistfiles = filterSources(files)
   linuxfiles = binarydistfiles[:]
   linuxfiles.append('boswars')
@@ -119,7 +122,7 @@ def buildPackages():
      winfiles = binarydistfiles[:]
      winfiles.append('boswars.exe')
      createZipArchive(name % 'win32', winfiles)
-  
+
 
 if __name__ == '__main__':
   buildPackages()
